/*
  MakerCloudMQTT example with using ESP8266WiFi library.
  e.g. ESP8266, WeMos D1...
*/

#include <ESP8266WiFi.h>
#include <MakerCloudMQTT.h>

// Change the credentials below, so your ESP8266 connects to your router
const char* ssid = "ssid";
const char* password = "password";

// Initializes the espClient.
WiFiClient espClient;
MakerCloudMQTT MakerCloudClient(espClient);

// MakerCloud MQTT Setting
// Copy and paste topic name from MakerCloud
String topic = "";

// Timers auxiliar variables
long now = millis();
long lastMeasure = 0;

// This functions connects your ESP8266 to your router
void setup_wifi() {
  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("WiFi connected - ESP IP address: ");
  Serial.println(WiFi.localIP());
}

// Subscribe Message Callback Function(if necessary)
void message_callback(String topic, String message) {
  Serial.print("Topic: ");
  Serial.print(topic);
  Serial.print("; Message:");
  Serial.println(message);
}

// Subscribe Key-Message Callback Function(if necessary)
void key_message_callback(String topic, String key, String message) {
  Serial.print("Topic: ");
  Serial.print(topic);
  Serial.print("; Key:");
  Serial.print(key);
  Serial.print("; Message:");
  Serial.println(message);
}

// Subscribe Key-Value Callback Function(if necessary)
void key_value_callback(String topic, String key, int value) {
  Serial.print("Topic: ");
  Serial.print(topic);
  Serial.print("; Key:");
  Serial.print(key);
  Serial.print("; Value:");
  Serial.println(value);
}

void setup() {
  Serial.begin(115200);
  randomSeed(analogRead(0));

  // MakerCloudMQTT Configuration
  MakerCloudClient.setUsername("Guest");
  // Enable to print extra log
  MakerCloudClient.setLog(true);

  // Connect ESP8266 to router
  setup_wifi();

  // Set callback function with different data type if need to subscribe topics
  MakerCloudClient.setMessageCallback(message_callback);
  MakerCloudClient.setKeyMessageCallback(key_message_callback);
  MakerCloudClient.setKeyValueCallback(key_value_callback);

  // Connect to MakerCloud
  if (MakerCloudClient.connect()) {
    // Subscribe to a topic
    MakerCloudClient.subscribe(topic);
  }
}

// The looping function will allow sending random number to MakerCloud
void loop() {
// Update user status and process 1 incoming MQTT message
  MakerCloudClient.listen();

  now = millis();
  // Publishes new random number every 5 seconds
  if (now - lastMeasure > 5000) {
    lastMeasure = now;

    // Publish Key-Value Pair
    int randNumber = random(0, 9);
    MakerCloudClient.publish_key_value(topic, "num", randNumber);

//    // Examples for other publishing method
//    // Publish Message
//    MakerCloudClient.publish_message(topic, "hello from arduino");
//    // Publish Key-Message Pair
//    MakerCloudClient.publish_key_message(topic, "command", "go");
//    // Publish Coordinate(lat, lng)
//    MakerCloudClient.publish_coordination(topic, "22.396427", "114.109497");
  }
}
